@@ -31,21 +31,21 @@ GType
 empathy_chat_view_get_type (void)
 {
 	static GType type = 0;
-	
+
 	if (!type) {
 		static const GTypeInfo type_info = {
 			sizeof (EmpathyChatViewIface),
 			chat_view_base_init,
 			NULL,
 		};
-		
+
 		type = g_type_register_static (G_TYPE_INTERFACE,
 					       "EmpathyChatView",
 					       &type_info, 0);
-		
+
 		g_type_interface_add_prerequisite (type, GTK_TYPE_WIDGET);
 	}
-	
+
 	return type;
 }
 
@@ -53,7 +53,7 @@ static void
 chat_view_base_init (gpointer klass)
 {
 	static gboolean initialized = FALSE;
-	
+
 	if (!initialized) {
 		initialized = TRUE;
 	}
@@ -64,7 +64,7 @@ empathy_chat_view_append_message (EmpathyChatView *view,
 				  EmpathyMessage  *msg)
 {
 	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-	
+
 	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->append_message) {
 		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->append_message (view,
 									 msg);
@@ -76,19 +76,31 @@ empathy_chat_view_append_event (EmpathyChatView *view,
 				const gchar    *str)
 {
 	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-	
+
 	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->append_event) {
 		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->append_event (view,
 								       str);
 	}
 }
 
+void
+empathy_chat_view_edit_message (EmpathyChatView *view,
+				EmpathyMessage  *message)
+{
+	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
+
+	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->edit_message) {
+		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->edit_message (
+			view, message);
+	}
+}
+
 void
 empathy_chat_view_scroll (EmpathyChatView *view,
 			  gboolean        allow_scrolling)
 {
 	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-	
+
 	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->scroll) {
 		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->scroll (view,
 								 allow_scrolling);
@@ -99,7 +111,7 @@ void
 empathy_chat_view_scroll_down (EmpathyChatView *view)
 {
 	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-	
+
 	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->scroll_down) {
 		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->scroll_down (view);
 	}
@@ -109,7 +121,7 @@ gboolean
 empathy_chat_view_get_has_selection (EmpathyChatView *view)
 {
 	g_return_val_if_fail (EMPATHY_IS_CHAT_VIEW (view), FALSE);
-	
+
 	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->get_has_selection) {
 		return EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->get_has_selection (view);
 	}
@@ -120,7 +132,7 @@ void
 empathy_chat_view_clear (EmpathyChatView *view)
 {
 	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-	
+
 	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->clear) {
 		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->clear (view);
 	}
@@ -129,14 +141,16 @@ empathy_chat_view_clear (EmpathyChatView *view)
 gboolean
 empathy_chat_view_find_previous (EmpathyChatView *view,
 				 const gchar    *search_criteria,
-				 gboolean        new_search)
+				 gboolean        new_search,
+				 gboolean        match_case)
 {
 	g_return_val_if_fail (EMPATHY_IS_CHAT_VIEW (view), FALSE);
-	
+
 	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->find_previous) {
 		return EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->find_previous (view,
 									       search_criteria,
-									       new_search);
+									       new_search,
+									       match_case);
 	}
 	return FALSE;
 }
@@ -144,14 +158,16 @@ empathy_chat_view_find_previous (EmpathyChatView *view,
 gboolean
 empathy_chat_view_find_next (EmpathyChatView *view,
 			     const gchar    *search_criteria,
-			     gboolean        new_search)
+			     gboolean        new_search,
+			     gboolean        match_case)
 {
 	g_return_val_if_fail (EMPATHY_IS_CHAT_VIEW (view), FALSE);
-	
+
 	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->find_next) {
 		return EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->find_next (view,
 									   search_criteria,
-									   new_search);
+									   new_search,
+									   match_case);
 	}
 	return FALSE;
 }
@@ -160,14 +176,16 @@ empathy_chat_view_find_next (EmpathyChatView *view,
 void
 empathy_chat_view_find_abilities (EmpathyChatView *view,
 				  const gchar    *search_criteria,
+				  gboolean        match_case,
 				  gboolean       *can_do_previous,
 				  gboolean       *can_do_next)
 {
 	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-	
+
 	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->find_abilities) {
 		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->find_abilities (view,
 									 search_criteria,
+									 match_case,
 									 can_do_previous,
 									 can_do_next);
 	}
@@ -175,12 +193,13 @@ empathy_chat_view_find_abilities (EmpathyChatView *view,
 
 void
 empathy_chat_view_highlight (EmpathyChatView *view,
-			     const gchar     *text)
+			     const gchar     *text,
+			     gboolean         match_case)
 {
 	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-	
+
 	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->highlight) {
-		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->highlight (view, text);
+		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->highlight (view, text, match_case);
 	}
 }
 
@@ -188,9 +207,31 @@ void
 empathy_chat_view_copy_clipboard (EmpathyChatView *view)
 {
 	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
-	
+
 	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->copy_clipboard) {
 		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->copy_clipboard (view);
 	}
 }
 
+void
+empathy_chat_view_focus_toggled (EmpathyChatView *view,
+				 gboolean         has_focus)
+{
+	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
+
+	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->focus_toggled) {
+		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->focus_toggled (view, has_focus);
+	}
+}
+
+void
+empathy_chat_view_message_acknowledged (EmpathyChatView *view,
+				        EmpathyMessage  *message)
+{
+	g_return_if_fail (EMPATHY_IS_CHAT_VIEW (view));
+
+	if (EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->message_acknowledged) {
+		EMPATHY_TYPE_CHAT_VIEW_GET_IFACE (view)->message_acknowledged (view, message);
+	}
+}
+
