@@ -46,6 +46,8 @@ struct _EmpathyChatViewIface {
 						  EmpathyMessage  *msg);
 	void             (*append_event)         (EmpathyChatView *view,
 						  const gchar     *str);
+	void             (*edit_message)         (EmpathyChatView *view,
+						  EmpathyMessage  *message);
 	void             (*scroll)               (EmpathyChatView *view,
 						  gboolean         allow_scrolling);
 	void             (*scroll_down)          (EmpathyChatView *view);
@@ -79,6 +81,8 @@ void             empathy_chat_view_append_message       (EmpathyChatView *view,
 							 EmpathyMessage  *msg);
 void             empathy_chat_view_append_event         (EmpathyChatView *view,
 							 const gchar     *str);
+void             empathy_chat_view_edit_message         (EmpathyChatView *view,
+							 EmpathyMessage  *message);
 void             empathy_chat_view_scroll               (EmpathyChatView *view,
 							 gboolean         allow_scrolling);
 void             empathy_chat_view_scroll_down          (EmpathyChatView *view);
