@@ -50,7 +50,7 @@ typedef struct _EmpathyChatClass  EmpathyChatClass;
 typedef struct _EmpathyChatPriv   EmpathyChatPriv;
 
 struct _EmpathyChat {
-	GtkBin parent;
+	GtkBox parent;
 	EmpathyChatPriv *priv;
 
 	/* Protected */
@@ -59,7 +59,7 @@ struct _EmpathyChat {
 };
 
 struct _EmpathyChatClass {
-	GtkBinClass parent;
+	GtkBoxClass parent;
 };
 
 GType              empathy_chat_get_type             (void);
@@ -69,7 +69,7 @@ void               empathy_chat_set_tp_chat          (EmpathyChat   *chat,
 						      EmpathyTpChat *tp_chat);
 TpAccount *        empathy_chat_get_account          (EmpathyChat   *chat);
 const gchar *      empathy_chat_get_id               (EmpathyChat   *chat);
-const gchar *      empathy_chat_get_name             (EmpathyChat   *chat);
+gchar *            empathy_chat_dup_name             (EmpathyChat   *chat);
 const gchar *      empathy_chat_get_subject          (EmpathyChat   *chat);
 EmpathyContact *   empathy_chat_get_remote_contact   (EmpathyChat   *chat);
 GtkWidget *        empathy_chat_get_contact_menu     (EmpathyChat   *chat);
@@ -83,12 +83,24 @@ void               empathy_chat_correct_word         (EmpathyChat   *chat,
 						      GtkTextIter   *start,
 						      GtkTextIter   *end,
 						      const gchar   *new_word);
+void               empathy_chat_join_muc             (EmpathyChat   *chat,
+						      const gchar   *room);
 gboolean           empathy_chat_is_room              (EmpathyChat   *chat);
 void               empathy_chat_set_show_contacts    (EmpathyChat *chat,
                                                       gboolean     show);
 guint              empathy_chat_get_nb_unread_messages (EmpathyChat   *chat);
 
 void               empathy_chat_messages_read        (EmpathyChat *self);
+
+gboolean           empathy_chat_is_composing (EmpathyChat *chat);
+
+gboolean           empathy_chat_is_sms_channel       (EmpathyChat *self);
+guint              empathy_chat_get_n_messages_sending (EmpathyChat *self);
+
+gchar *            empathy_chat_dup_text             (EmpathyChat *self);
+void               empathy_chat_set_text             (EmpathyChat *self,
+                                                      const gchar *text);
+
 G_END_DECLS
 
 #endif /* __EMPATHY_CHAT_H__ */
