@@ -36,6 +36,9 @@ G_BEGIN_DECLS
 #define EMPATHY_AV_BUS_NAME_SUFFIX "Empathy.AudioVideo"
 #define EMPATHY_AV_BUS_NAME TP_CLIENT_BUS_NAME_BASE EMPATHY_AV_BUS_NAME_SUFFIX
 
+#define EMPATHY_CALL_BUS_NAME_SUFFIX "Empathy.Call"
+#define EMPATHY_CALL_BUS_NAME TP_CLIENT_BUS_NAME_BASE EMPATHY_CALL_BUS_NAME_SUFFIX
+
 #define EMPATHY_FT_BUS_NAME_SUFFIX "Empathy.FileTransfer"
 #define EMPATHY_FT_BUS_NAME TP_CLIENT_BUS_NAME_BASE EMPATHY_FT_BUS_NAME_SUFFIX
 
