@@ -1375,6 +1375,10 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 			notify_notification_set_hint_string (notification,
 				EMPATHY_NOTIFY_MANAGER_CAP_X_CANONICAL_APPEND, "1");
 		}
+
+		notify_notification_set_hint (notification,
+			EMPATHY_NOTIFY_MANAGER_CAP_CATEGORY,
+			g_variant_new_string ("im.received"));
 	}
 
 	pixbuf = empathy_notify_manager_get_pixbuf_for_notification (priv->notify_mgr,
