@@ -59,7 +59,8 @@ struct _EmpathyChatWindowClass {
 GType              empathy_chat_window_get_type       (void);
 
 EmpathyChat *      empathy_chat_window_find_chat      (TpAccount        *account,
-						       const gchar      *id);
+						       const gchar      *id,
+						       gboolean          sms_channel);
 void               empathy_chat_window_present_chat   (EmpathyChat      *chat,
 						       gint64 timestamp);
 
