@@ -437,7 +437,7 @@ account_manager_prepared_cb (GObject *source_object,
 	EmpathyStatusIcon *icon = user_data;
 	GError *error = NULL;
 
-	if (!tp_account_manager_prepare_finish (account_manager, result, &error)) {
+	if (!tp_proxy_prepare_finish (account_manager, result, &error)) {
 		DEBUG ("Failed to prepare account manager: %s", error->message);
 		g_error_free (error);
 		return;
@@ -465,7 +465,7 @@ empathy_status_icon_init (EmpathyStatusIcon *icon)
 	priv->account_manager = tp_account_manager_dup ();
 	priv->event_manager = empathy_event_manager_dup_singleton ();
 
-	tp_account_manager_prepare_async (priv->account_manager, NULL,
+	tp_proxy_prepare_async (priv->account_manager, NULL,
 	    account_manager_prepared_cb, icon);
 
 	/* make icon listen and respond to MAIN_WINDOW_HIDDEN changes */
