@@ -82,6 +82,9 @@ struct _TpyCallChannelPrivate
   GPtrArray *contents;
 
   GSimpleAsyncResult *result;
+
+  gboolean properties_retrieved;
+  gboolean ready;
 };
 
 enum /* props */
@@ -98,7 +101,8 @@ enum /* props */
   PROP_INITIAL_AUDIO_NAME,
   PROP_INITIAL_VIDEO,
   PROP_INITIAL_VIDEO_NAME,
-  PROP_MUTABLE_CONTENTS
+  PROP_MUTABLE_CONTENTS,
+  PROP_READY
 };
 
 enum /* signals */
@@ -112,6 +116,68 @@ enum /* signals */
 
 static guint _signals[LAST_SIGNAL] = { 0, };
 
+static void
+update_call_members (TpyCallChannel *self,
+    GHashTable *flags_changed,
+    const GArray *removed)
+{
+  GHashTableIter iter;
+  gpointer key, value;
+  TpHandle handle;
+  guint i;
+
+  if (flags_changed != NULL)
+    {
+      g_hash_table_iter_init (&iter, flags_changed);
+      while (g_hash_table_iter_next (&iter, &key, &value))
+        g_hash_table_insert (self->priv->members, key, value);
+    }
+
+  if (removed != NULL)
+    {
+      for (i = 0; i < removed->len; i++)
+        {
+          handle = g_array_index (removed, TpHandle, i);
+          g_hash_table_remove (self->priv->members, GUINT_TO_POINTER (handle));
+        }
+    }
+}
+
+static void
+maybe_go_to_ready (TpyCallChannel *self)
+{
+  TpyCallChannelPrivate *priv = self->priv;
+  guint i;
+
+  if (priv->ready)
+    return;
+
+  if (!priv->properties_retrieved)
+    return;
+
+  for (i = 0 ; i < priv->contents->len; i++)
+    {
+      TpyCallContent *c = g_ptr_array_index (self->priv->contents, i);
+      gboolean ready;
+
+      g_object_get (c, "ready", &ready, NULL);
+
+      if (!ready)
+        return;
+    }
+
+  priv->ready = TRUE;
+  g_object_notify (G_OBJECT (self), "ready");
+}
+
+static void
+on_content_ready_cb (TpyCallContent *content,
+  GParamSpec *spec,
+  TpyCallChannel *self)
+{
+  maybe_go_to_ready (self);
+}
+
 static void
 on_content_added_cb (TpProxy *proxy,
     const gchar *content_path,
@@ -137,6 +203,8 @@ on_content_added_cb (TpProxy *proxy,
     }
 
   g_ptr_array_add (self->priv->contents, content);
+  tp_g_signal_connect_object (content, "notify::ready",
+    G_CALLBACK (on_content_ready_cb), self, 0);
 
   g_signal_emit (self, _signals[CONTENT_ADDED], 0, content);
 }
@@ -183,11 +251,11 @@ on_call_members_changed_cb (TpProxy *proxy,
 {
   TpyCallChannel *self = TPY_CALL_CHANNEL (proxy);
 
-  DEBUG ("Call members changed: %d members",
-      g_hash_table_size (flags_changed));
+  DEBUG ("Call members: %d changed, %d removed",
+      g_hash_table_size (flags_changed),
+      removed->len);
 
-  g_hash_table_unref (self->priv->members);
-  self->priv->members = g_hash_table_ref (flags_changed);
+  update_call_members (self, flags_changed, removed);
 
   g_signal_emit (self, _signals[MEMBERS_CHANGED], 0, self->priv->members);
 }
@@ -250,9 +318,7 @@ on_call_channel_get_all_properties_cb (TpProxy *proxy,
 
   hash_table = tp_asv_get_boxed (properties,
       "CallMembers", TPY_HASH_TYPE_CALL_MEMBER_MAP);
-  if (hash_table != NULL)
-    self->priv->members = g_boxed_copy (TPY_HASH_TYPE_CALL_MEMBER_MAP,
-        hash_table);
+  update_call_members (self, hash_table, NULL);
 
   contents = tp_asv_get_boxed (properties,
       "Contents", TP_ARRAY_TYPE_OBJECT_PATH_LIST);
@@ -278,9 +344,16 @@ on_call_channel_get_all_properties_cb (TpProxy *proxy,
         }
 
       g_ptr_array_add (self->priv->contents, content);
+
+      tp_g_signal_connect_object (content, "notify::ready",
+        G_CALLBACK (on_content_ready_cb), self, 0);
     }
 
   g_signal_emit (self, _signals[MEMBERS_CHANGED], 0, self->priv->members);
+
+  self->priv->properties_retrieved = TRUE;
+
+  maybe_go_to_ready (self);
 }
 
 static void
@@ -290,6 +363,7 @@ tpy_call_channel_dispose (GObject *obj)
 
   tp_clear_pointer (&self->priv->contents, g_ptr_array_unref);
   tp_clear_pointer (&self->priv->details, g_hash_table_unref);
+  tp_clear_pointer (&self->priv->members, g_hash_table_unref);
 
   tp_clear_object (&self->priv->result);
 
@@ -334,6 +408,10 @@ tpy_call_channel_get_property (GObject *object,
         g_value_set_boolean (value, self->priv->initial_video);
         break;
 
+      case PROP_READY:
+        g_value_set_boolean (value, self->priv->ready);
+        break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
         break;
@@ -521,6 +599,21 @@ tpy_call_channel_class_init (TpyCallChannelClass *klass)
   g_object_class_install_property (gobject_class,
       PROP_INITIAL_VIDEO, param_spec);
 
+  /**
+   * TpyCallChannel:ready:
+   *
+   * Whether or call channel got all its async information
+   *
+   * Since:
+   */
+  param_spec = g_param_spec_boolean ("ready", "Ready",
+      "If true the call channel and all its contents have retrieved all "
+      "all async information from the CM",
+      FALSE,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (gobject_class, PROP_READY,
+      param_spec);
+
   /**
    * TpyCallChannel::content-added
    * @self: the #TpyCallChannel
@@ -570,13 +663,12 @@ tpy_call_channel_class_init (TpyCallChannelClass *klass)
       0, NULL, NULL,
       _tpy_marshal_VOID__UINT_UINT_BOXED_BOXED,
       G_TYPE_NONE,
-      4, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_PTR_ARRAY, G_TYPE_HASH_TABLE);
+      4, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_VALUE_ARRAY, G_TYPE_HASH_TABLE);
 
   /**
    * TpyCallChannel::members-changed
    * @self: the #TpyCallChannel
-   * @state: the new #TpyCallState
-   * @flags: the new #TpyCallFlags
+   * @members: the call members
    *
    * The ::members-changed signal is emitted whenever participants
    * are added, removed, or their flags change.
@@ -588,6 +680,7 @@ tpy_call_channel_class_init (TpyCallChannelClass *klass)
       g_cclosure_marshal_VOID__BOXED,
       G_TYPE_NONE,
       1, G_TYPE_HASH_TABLE);
+
 }
 
 static void
@@ -597,6 +690,8 @@ tpy_call_channel_init (TpyCallChannel *self)
       TPY_TYPE_CALL_CHANNEL, TpyCallChannelPrivate);
 
   self->priv->contents = g_ptr_array_new_with_free_func (g_object_unref);
+
+  self->priv->members = g_hash_table_new (g_direct_hash, g_direct_equal);
 }
 
 /**
@@ -833,6 +928,41 @@ tpy_call_channel_send_video (TpyCallChannel *self,
         NULL, NULL, NULL, NULL);
 }
 
+TpySendingState
+tpy_call_channel_get_video_state (TpyCallChannel *self)
+{
+  TpySendingState result = TPY_SENDING_STATE_NONE;
+  guint i;
+
+  g_return_val_if_fail (TPY_IS_CALL_CHANNEL (self), TPY_SENDING_STATE_NONE);
+
+  for (i = 0 ; i < self->priv->contents->len ; i++)
+    {
+      TpyCallContent *content = g_ptr_array_index (self->priv->contents, i);
+
+      if (tpy_call_content_get_media_type (content)
+          == TP_MEDIA_STREAM_TYPE_VIDEO)
+        {
+          GList *l;
+
+          for (l = tpy_call_content_get_streams (content);
+              l != NULL ; l = g_list_next (l))
+            {
+              TpyCallStream *stream = TPY_CALL_STREAM (l->data);
+              TpySendingState state;
+
+              g_object_get (stream, "local-sending-state", &state, NULL);
+              if (state != TPY_SENDING_STATE_PENDING_STOP_SENDING &&
+                  state > result)
+                result = state;
+            }
+        }
+    }
+
+  return result;
+}
+
+
 gboolean
 tpy_call_channel_has_dtmf (TpyCallChannel *self)
 {
